"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findFillBlock = exports.relayFilledAmount = exports.getRelayDataHash = exports.getDepositIdAtBlock = exports.getBlockRangeForDepositId = void 0;
var tslib_1 = require("tslib");
var assert_1 = tslib_1.__importDefault(require("assert"));
var ethers_1 = require("ethers");
var interfaces_1 = require("../interfaces");
var BigNumberUtils_1 = require("./BigNumberUtils");
var TypeGuards_1 = require("./TypeGuards");
var V3Utils_1 = require("./V3Utils");
var NetworkUtils_1 = require("./NetworkUtils");
function getBlockRangeForDepositId(targetDepositId, initLow, initHigh, maxSearches, spokePool) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var deploymentBlock, queriedIds, _getDepositIdAtBlock, _a, highestDepositIdInRange, lowestDepositIdInRange, low, high, searches, mid, midDepositId, accountedIdByMidBlock;
        var _this = this;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    deploymentBlock = spokePool.deploymentBlock;
                    initHigh = Math.min(initHigh, spokePool.latestBlockSearched);
                    [
                        [spokePool.isUpdated, "Spoke pool client is not updated"],
                        [initLow <= initHigh, "Binary search failed because low > high"],
                        [initLow >= deploymentBlock, "Binary search failed because low must be >= deploymentBlock"],
                        [maxSearches > 0, "maxSearches must be > 0"],
                        [deploymentBlock >= 0, "deploymentBlock must be >= 0"],
                    ].forEach(function (_a) {
                        var condition = _a[0], errorMessage = _a[1];
                        if (!condition) {
                            throw new Error(errorMessage);
                        }
                    });
                    queriedIds = {};
                    _getDepositIdAtBlock = function (blockNumber) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        var _a, _b, _c;
                        var _d;
                        return tslib_1.__generator(this, function (_e) {
                            switch (_e.label) {
                                case 0:
                                    if (!((_d = queriedIds[blockNumber]) !== null && _d !== void 0)) return [3, 1];
                                    _a = _d;
                                    return [3, 3];
                                case 1:
                                    _b = queriedIds;
                                    _c = blockNumber;
                                    return [4, spokePool._getDepositIdAtBlock(blockNumber)];
                                case 2:
                                    _a = (_b[_c] = _e.sent());
                                    _e.label = 3;
                                case 3:
                                    _a;
                                    return [2, queriedIds[blockNumber]];
                            }
                        });
                    }); };
                    return [4, Promise.all([
                            _getDepositIdAtBlock(initHigh),
                            _getDepositIdAtBlock(Math.max(deploymentBlock, initLow - 1)),
                        ])];
                case 1:
                    _a = _b.sent(), highestDepositIdInRange = _a[0], lowestDepositIdInRange = _a[1];
                    if (highestDepositIdInRange <= targetDepositId) {
                        throw new Error("Target depositId is greater than the initial high block (".concat(targetDepositId, " > ").concat(highestDepositIdInRange, ")"));
                    }
                    if (lowestDepositIdInRange > targetDepositId) {
                        throw new Error("Target depositId is less than the initial low block (".concat(targetDepositId, " > ").concat(lowestDepositIdInRange, ")"));
                    }
                    low = initLow;
                    high = initHigh;
                    searches = 0;
                    _b.label = 2;
                case 2:
                    mid = Math.floor((low + high) / 2);
                    return [4, _getDepositIdAtBlock(mid)];
                case 3:
                    midDepositId = _b.sent();
                    accountedIdByMidBlock = midDepositId - 1;
                    if (targetDepositId <= accountedIdByMidBlock) {
                        high = mid;
                    }
                    else {
                        low = mid + 1;
                    }
                    _b.label = 4;
                case 4:
                    if (++searches <= maxSearches && low < high) return [3, 2];
                    _b.label = 5;
                case 5:
                    if (low > high) {
                        throw new Error("Binary search failed (".concat(low, " > ").concat(high, "). SHOULD NEVER HAPPEN (but here we are)"));
                    }
                    return [2, { low: low, high: high }];
            }
        });
    });
}
exports.getBlockRangeForDepositId = getBlockRangeForDepositId;
function getDepositIdAtBlock(contract, blockTag) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var depositIdAtBlock;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, contract.numberOfDeposits({ blockTag: blockTag })];
                case 1:
                    depositIdAtBlock = _a.sent();
                    if (!Number.isInteger(depositIdAtBlock) || depositIdAtBlock < 0) {
                        throw new Error("Invalid deposit count");
                    }
                    return [2, depositIdAtBlock];
            }
        });
    });
}
exports.getDepositIdAtBlock = getDepositIdAtBlock;
function getRelayDataHash(relayData, destinationChainId) {
    if ((0, V3Utils_1.isV2RelayData)(relayData)) {
        (0, assert_1.default)(!(0, TypeGuards_1.isDefined)(destinationChainId) || destinationChainId === relayData.destinationChainId);
        return getV2RelayHash(relayData);
    }
    (0, assert_1.default)((0, TypeGuards_1.isDefined)(destinationChainId));
    return getV3RelayHash(relayData, destinationChainId);
}
exports.getRelayDataHash = getRelayDataHash;
function getV2RelayHash(relayData) {
    return ethers_1.utils.keccak256(ethers_1.utils.defaultAbiCoder.encode([
        "tuple(" +
            "address depositor," +
            "address recipient," +
            "address destinationToken," +
            "uint256 amount," +
            "uint256 originChainId," +
            "uint256 destinationChainId," +
            "int64 realizedLpFeePct," +
            "int64 relayerFeePct," +
            "uint32 depositId," +
            "bytes message" +
            ")",
    ], [relayData]));
}
function getV3RelayHash(relayData, destinationChainId) {
    return ethers_1.utils.keccak256(ethers_1.utils.defaultAbiCoder.encode([
        "tuple(" +
            "address depositor," +
            "address recipient," +
            "address exclusiveRelayer," +
            "address inputToken," +
            "address outputToken," +
            "uint256 inputAmount," +
            "uint256 outputAmount," +
            "uint256 originChainId," +
            "uint32 depositId," +
            "uint32 fillDeadline," +
            "uint32 exclusivityDeadline," +
            "bytes message" +
            ")",
        "uint256 destinationChainId",
    ], [relayData, destinationChainId]));
}
function relayFilledAmount(spokePool, relayData, blockTag) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var hash, fillStatus;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    hash = getRelayDataHash(relayData);
                    if ((0, V3Utils_1.isV2RelayData)(relayData)) {
                        return [2, spokePool.relayFills(hash, { blockTag: blockTag })];
                    }
                    return [4, spokePool.fillStatuses(hash, { blockTag: blockTag })];
                case 1:
                    fillStatus = _a.sent();
                    return [2, fillStatus === interfaces_1.FillStatus.Filled ? relayData.outputAmount : BigNumberUtils_1.bnZero];
            }
        });
    });
}
exports.relayFilledAmount = relayFilledAmount;
function findFillBlock(spokePool, relayData, lowBlockNumber, highBlockNumber) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var provider, _a, destinationChainId, _b, initialFillAmount, finalFillAmount, relayAmount, depositId, originChainId, _c, srcChain, dstChain, midBlockNumber, filledAmount;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    provider = spokePool.provider;
                    if (!(highBlockNumber !== null && highBlockNumber !== void 0)) return [3, 1];
                    _a = highBlockNumber;
                    return [3, 3];
                case 1: return [4, provider.getBlockNumber()];
                case 2:
                    _a = (highBlockNumber = _d.sent());
                    _d.label = 3;
                case 3:
                    _a;
                    (0, assert_1.default)(highBlockNumber > lowBlockNumber, "Block numbers out of range (".concat(lowBlockNumber, " > ").concat(highBlockNumber, ")"));
                    return [4, provider.getNetwork()];
                case 4:
                    destinationChainId = (_d.sent()).chainId;
                    return [4, Promise.all([
                            relayFilledAmount(spokePool, relayData, lowBlockNumber),
                            relayFilledAmount(spokePool, relayData, highBlockNumber),
                        ])];
                case 5:
                    _b = _d.sent(), initialFillAmount = _b[0], finalFillAmount = _b[1];
                    relayAmount = (0, V3Utils_1.getRelayDataOutputAmount)(relayData);
                    if (finalFillAmount.lt(relayAmount)) {
                        return [2, undefined];
                    }
                    if (initialFillAmount.eq(relayAmount)) {
                        depositId = relayData.depositId, originChainId = relayData.originChainId;
                        _c = [(0, NetworkUtils_1.getNetworkName)(originChainId), (0, NetworkUtils_1.getNetworkName)(destinationChainId)], srcChain = _c[0], dstChain = _c[1];
                        throw new Error("".concat(srcChain, " deposit ").concat(depositId, " filled on ").concat(dstChain, " before block ").concat(lowBlockNumber));
                    }
                    _d.label = 6;
                case 6:
                    midBlockNumber = Math.floor((highBlockNumber + lowBlockNumber) / 2);
                    return [4, relayFilledAmount(spokePool, relayData, midBlockNumber)];
                case 7:
                    filledAmount = _d.sent();
                    if (filledAmount.eq(relayAmount)) {
                        highBlockNumber = midBlockNumber;
                    }
                    else {
                        lowBlockNumber = midBlockNumber + 1;
                    }
                    _d.label = 8;
                case 8:
                    if (lowBlockNumber < highBlockNumber) return [3, 6];
                    _d.label = 9;
                case 9: return [2, lowBlockNumber];
            }
        });
    });
}
exports.findFillBlock = findFillBlock;
//# sourceMappingURL=SpokeUtils.js.map