"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSlowFillLeafChainId = exports.getRelayDataOutputAmount = exports.getRelayDataOutputToken = exports.getTotalFilledAmount = exports.getFillAmount = exports.getFillOutputAmount = exports.getDepositOutputAmount = exports.getDepositInputAmount = exports.getFillOutputToken = exports.getDepositOutputToken = exports.getDepositInputToken = exports.isV3SlowFillLeaf = exports.isV2SlowFillLeaf = exports.isSlowFill = exports.isV3RelayData = exports.isV2RelayData = exports.isV3Fill = exports.isV2Fill = exports.isV3SpeedUp = exports.isV2SpeedUp = exports.isV3Deposit = exports.isV2Deposit = exports.isV3 = exports.V3_MIN_CONFIG_STORE_VERSION = void 0;
var interfaces_1 = require("../interfaces");
var TypeGuards_1 = require("./TypeGuards");
exports.V3_MIN_CONFIG_STORE_VERSION = 3;
function isV3(version) {
    return version >= exports.V3_MIN_CONFIG_STORE_VERSION;
}
exports.isV3 = isV3;
function isV2Deposit(deposit) {
    return (0, TypeGuards_1.isDefined)(deposit.originToken);
}
exports.isV2Deposit = isV2Deposit;
function isV3Deposit(deposit) {
    return (0, TypeGuards_1.isDefined)(deposit.inputToken);
}
exports.isV3Deposit = isV3Deposit;
function isV2SpeedUp(speedUp) {
    return (0, TypeGuards_1.isDefined)(speedUp.newRelayerFeePct);
}
exports.isV2SpeedUp = isV2SpeedUp;
function isV3SpeedUp(speedUp) {
    return (0, TypeGuards_1.isDefined)(speedUp.updatedOutputAmount);
}
exports.isV3SpeedUp = isV3SpeedUp;
function isV2Fill(fill) {
    return (0, TypeGuards_1.isDefined)(fill.destinationToken);
}
exports.isV2Fill = isV2Fill;
function isV3Fill(fill) {
    return (0, TypeGuards_1.isDefined)(fill.inputToken);
}
exports.isV3Fill = isV3Fill;
function isV2RelayData(relayData) {
    return (0, TypeGuards_1.isDefined)(relayData.destinationToken);
}
exports.isV2RelayData = isV2RelayData;
function isV3RelayData(relayData) {
    return (0, TypeGuards_1.isDefined)(relayData.outputToken);
}
exports.isV3RelayData = isV3RelayData;
function isSlowFill(fill) {
    return isV2Fill(fill) ? fill.updatableRelayData.isSlowRelay : fill.updatableRelayData.fillType === interfaces_1.FillType.SlowFill;
}
exports.isSlowFill = isSlowFill;
function isV2SlowFillLeaf(slowFillLeaf) {
    return (0, TypeGuards_1.isDefined)(slowFillLeaf.payoutAdjustmentPct) && isV2RelayData(slowFillLeaf.relayData);
}
exports.isV2SlowFillLeaf = isV2SlowFillLeaf;
function isV3SlowFillLeaf(slowFillLeaf) {
    return (0, TypeGuards_1.isDefined)(slowFillLeaf.updatedOutputAmount) && isV3RelayData(slowFillLeaf.relayData);
}
exports.isV3SlowFillLeaf = isV3SlowFillLeaf;
function getDepositInputToken(deposit) {
    return isV2Deposit(deposit) ? deposit.originToken : deposit.inputToken;
}
exports.getDepositInputToken = getDepositInputToken;
function getDepositOutputToken(deposit) {
    return isV2Deposit(deposit) ? deposit.destinationToken : deposit.outputToken;
}
exports.getDepositOutputToken = getDepositOutputToken;
function getFillOutputToken(fill) {
    return isV2Fill(fill) ? fill.destinationToken : fill.outputToken;
}
exports.getFillOutputToken = getFillOutputToken;
function getDepositInputAmount(deposit) {
    return isV2Deposit(deposit) ? deposit.amount : deposit.inputAmount;
}
exports.getDepositInputAmount = getDepositInputAmount;
function getDepositOutputAmount(deposit) {
    return isV2Deposit(deposit) ? deposit.amount : deposit.outputAmount;
}
exports.getDepositOutputAmount = getDepositOutputAmount;
function getFillOutputAmount(fill) {
    return isV2Fill(fill) ? fill.amount : fill.outputAmount;
}
exports.getFillOutputAmount = getFillOutputAmount;
function getFillAmount(fill) {
    return isV2Fill(fill) ? fill.fillAmount : fill.outputAmount;
}
exports.getFillAmount = getFillAmount;
function getTotalFilledAmount(fill) {
    return isV2Fill(fill) ? fill.totalFilledAmount : fill.outputAmount;
}
exports.getTotalFilledAmount = getTotalFilledAmount;
function getRelayDataOutputToken(relayData) {
    return isV2RelayData(relayData) ? relayData.destinationToken : relayData.outputToken;
}
exports.getRelayDataOutputToken = getRelayDataOutputToken;
function getRelayDataOutputAmount(relayData) {
    return isV2RelayData(relayData) ? relayData.amount : relayData.outputAmount;
}
exports.getRelayDataOutputAmount = getRelayDataOutputAmount;
function getSlowFillLeafChainId(leaf) {
    return isV2SlowFillLeaf(leaf) ? leaf.relayData.destinationChainId : leaf.chainId;
}
exports.getSlowFillLeafChainId = getSlowFillLeafChainId;
//# sourceMappingURL=V3Utils.js.map