import { BigNumber, Contract, Event, providers } from "ethers";
import winston from "winston";
import { DepositWithBlock, FundsDepositedEvent, RealizedLpFee, SlowFillRequestWithBlock, v2DepositWithBlock, v2FillWithBlock, v2SpeedUp, v3DepositWithBlock, v3FillWithBlock, v3SpeedUp } from "../../interfaces";
import { SpokePoolClient, SpokePoolUpdate } from "../SpokePoolClient";
import { EventManager, EventOverrides } from "./MockEvents";
type Block = providers.Block;
export declare class MockSpokePoolClient extends SpokePoolClient {
    eventManager: EventManager;
    private realizedLpFeePct;
    private realizedLpFeePctOverride;
    private destinationTokenForChainOverride;
    depositIdAtBlock: number[];
    numberOfDeposits: number;
    blocks: Record<number, Block>;
    constructor(logger: winston.Logger, spokePool: Contract, chainId: number, deploymentBlock: number);
    setDefaultRealizedLpFeePct(fee: BigNumber): void;
    clearDefaultRealizedLpFeePct(): void;
    computeRealizedLpFeePct(depositEvent: FundsDepositedEvent): Promise<RealizedLpFee>;
    batchComputeRealizedLpFeePct(depositEvents: FundsDepositedEvent[]): Promise<RealizedLpFee[]>;
    setDestinationTokenForChain(chainId: number, token: string): void;
    getDestinationTokenForDeposit(deposit: DepositWithBlock): string;
    setLatestBlockNumber(blockNumber: number): void;
    setDepositIds(_depositIds: number[]): void;
    _getDepositIdAtBlock(blockTag: number): Promise<number>;
    _update(eventsToQuery: string[]): Promise<SpokePoolUpdate>;
    readonly eventSignatures: Record<string, string>;
    deposit(deposit: v2DepositWithBlock): Event;
    depositV3(deposit: v3DepositWithBlock): Event;
    fillRelay(fill: v2FillWithBlock): Event;
    fillV3Relay(fill: v3FillWithBlock): Event;
    speedUpDeposit(speedUp: v2SpeedUp): Event;
    speedUpV3Deposit(speedUp: v3SpeedUp): Event;
    requestV3SlowFill(request: SlowFillRequestWithBlock): Event;
    setEnableRoute(originToken: string, destinationChainId: number, enabled: boolean, overrides?: EventOverrides): Event;
}
export {};
//# sourceMappingURL=MockSpokePoolClient.d.ts.map